import { generateUUID } from '@/libs/util';
import Flexbox from '@/components/flexbox';
import FlexboxItem from '@/components/flexbox-item';
import value2name from '@/filters/value2name';
import filter from 'array-filter';
import Cell from '@/components/cell';
import Popup from '@/components/popup';
import InlineDesc from '@/components/inline-desc';
import array2string from '@/filters/array2String';
import uuidMixin from '@/libs/mixin_uuid';
import fecha from '@/libs/fecha';
import name2value from '@/filters/name2value';

var time = Date.now || function () {
  return +new Date()
};

var running = {};
var counter = 1;
var desiredFrames = 60;
var millisecondsPerSecond = 1000;

// http://paulirish.com/2011/requestanimationframe-for-smart-animating/
// http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating

// requestAnimationFrame polyfill by Erik Möller
// fixes from Paul Irish and Tino Zijdel
if (typeof window !== 'undefined') {
  (function () {
    var lastTime = 0;
    var vendors = ['ms', 'moz', 'webkit', 'o'];
    for (var x = 0; x < vendors.length && !window.requestAnimationFrame; ++x) {
      window.requestAnimationFrame = window[vendors[x] + 'RequestAnimationFrame'];
      window.cancelAnimationFrame = window[vendors[x] + 'CancelAnimationFrame'] || window[vendors[x] + 'CancelRequestAnimationFrame'];
    }

    if (!window.requestAnimationFrame) {
      window.requestAnimationFrame = function (callback, element) {
        var currTime = new Date().getTime();
        var timeToCall = Math.max(0, 16 - (currTime - lastTime));
        var id = window.setTimeout(function () { callback(currTime + timeToCall); },
              timeToCall);
        lastTime = currTime + timeToCall;
        return id
      };
    }
    if (!window.cancelAnimationFrame) {
      window.cancelAnimationFrame = function (id) {
        clearTimeout(id);
      };
    }
  }());
}

var Animate = {

  // A requestAnimationFrame wrapper / polyfill.
  requestAnimationFrame: (function () {
    if (typeof window !== 'undefined') {
      var requestFrame = window.requestAnimationFrame;
      return function (callback, root) {
        requestFrame(callback, root);
      }
    }
  })(),

  // Stops the given animation.
  stop: function stop (id) {
    var cleared = running[id] != null;
    if (cleared) {
      running[id] = null;
    }
    return cleared
  },

  // Whether the given animation is still running.
  isRunning: function isRunning (id) {
    return running[id] != null
  },

  // Start the animation.
  start: function start (stepCallback, verifyCallback, completedCallback, duration, easingMethod, root) {
    var _this = this;
    var start = time();
    var lastFrame = start;
    var percent = 0;
    var dropCounter = 0;
    var id = counter++;

    if (!root) {
      root = document.body;
    }

    // Compacting running db automatically every few new animations
    if (id % 20 === 0) {
      var newRunning = {};
      for (var usedId in running) {
        newRunning[usedId] = true;
      }
      running = newRunning;
    }

    // This is the internal step method which is called every few milliseconds
    var step = function (virtual) {
      // Normalize virtual value
      var render = virtual !== true;
      // Get current time
      var now = time();

      // Verification is executed before next animation step
      if (!running[id] || (verifyCallback && !verifyCallback(id))) {
        running[id] = null;
        completedCallback && completedCallback(desiredFrames - (dropCounter / ((now - start) / millisecondsPerSecond)), id, false);
        return
      }

      // For the current rendering to apply let's update omitted steps in memory.
      // This is important to bring internal state variables up-to-date with progress in time.
      if (render) {
        var droppedFrames = Math.round((now - lastFrame) / (millisecondsPerSecond / desiredFrames)) - 1;
        for (var j = 0; j < Math.min(droppedFrames, 4); j++) {
          step(true);
          dropCounter++;
        }
      }

      // Compute percent value
      if (duration) {
        percent = (now - start) / duration;
        if (percent > 1) {
          percent = 1;
        }
      }

      // Execute step callback, then...
      var value = easingMethod ? easingMethod(percent) : percent;
      if ((stepCallback(value, now, render) === false || percent === 1) && render) {
        running[id] = null;
        completedCallback && completedCallback(desiredFrames - (dropCounter / ((now - start) / millisecondsPerSecond)), id, percent === 1 || duration == null);
      } else if (render) {
        lastFrame = now;
        _this.requestAnimationFrame(step, root);
      }
    };

    // Mark as running
    running[id] = true;
    // Init first step
    _this.requestAnimationFrame(step, root);
    // Return unique animation ID
    return id
  }
};

function getElement (expr) {
  return (typeof expr === 'string') ? document.querySelector(expr) : expr
}

function getComputedStyle (el, key) {
  var computedStyle = window.getComputedStyle(el);
  return computedStyle[key] || ''
}

// Easing Equations (c) 2003 Robert Penner, all rights reserved.
// Open source under the BSD License.
function easeOutCubic (pos) {
  return (Math.pow((pos - 1), 3) + 1)
}

function easeInOutCubic (pos) {
  if ((pos /= 0.5) < 1) {
    return 0.5 * Math.pow(pos, 3)
  }
  return 0.5 * (Math.pow((pos - 2), 3) + 2)
}

/*
 * Anima Scroller
 * Based Zynga Scroller (http://github.com/zynga/scroller)
 * Copyright 2011, Zynga Inc.
 * Licensed under the MIT License.
 * https://raw.github.com/zynga/scroller/master/MIT-LICENSE.txt
 */
var TEMPLATE = "\n<div class=\"scroller-component\" data-role=\"component\">\n  <div class=\"scroller-mask\" data-role=\"mask\"></div>\n  <div class=\"scroller-indicator\" data-role=\"indicator\"></div>\n  <div class=\"scroller-content\" data-role=\"content\"></div>\n</div>\n";

function encodeValue (value) {
  if (typeof value === 'number') {
    return value
  }
  return encodeURI(value)
}

function decodeValue (value) {
  var obj = JSON.parse(value);
  if (typeof obj.value === 'number') {
    return obj.value
  }
  return decodeURI(obj.value)
}

var Scroller = function (container, options) {
  var self = this;

  options = options || {};

  self.options = {
    itemClass: 'scroller-item',
    onStart: function onStart () {},
    onEnd: function onEnd () {},
    onSelect: function onSelect () {},
    defaultValue: 0,
    data: []
  };

  for (var key in options) {
    if (options[key] !== undefined) {
      self.options[key] = options[key];
    }
  }

  self.__container = getElement(container);

  var tempContainer = document.createElement('div');
  tempContainer.innerHTML = options.template || TEMPLATE;

  var component = self.__component = tempContainer.querySelector('[data-role=component]');
  var content = self.__content = component.querySelector('[data-role=content]');
  var indicator = component.querySelector('[data-role=indicator]');

  var data = self.options.data;
  var rangeKey = self.options.rangeKey;
  var html = '';
  if (data.length && data[0].constructor === Object) {
    data.forEach(function (row) {
      html += '<div class="' + self.options.itemClass + '" data-value=' + JSON.stringify({ value: encodeValue(row.value) }) + '>' + row[rangeKey] + '</div>';
    });
  } else {
    data.forEach(function (val) {
      html += '<div class="' + self.options.itemClass + '" data-value=' + JSON.stringify({ value: encodeValue(val) }) + '>' + val + '</div>';
    });
  }
  content.innerHTML = html;

  self.__container.appendChild(component);

  self.__itemHeight = parseFloat(getComputedStyle(indicator, 'height'), 10);

  self.__callback = options.callback || function (top) {
    content.style.webkitTransform = 'translate3d(0, ' + (-top) + 'px, 0)';
  };

  var rect = component.getBoundingClientRect();

  self.__clientTop = (rect.top + component.clientTop) || 0;

  self.__setDimensions(component.clientHeight, content.offsetHeight);

  if (component.clientHeight === 0) {
    self.__setDimensions(parseFloat(getComputedStyle(component, 'height'), 10), 204);
  }
  self.select(self.options.defaultValue, false);

  var touchStartHandler = function (e) {
    if (e.target.tagName.match(/input|textarea|select/i)) {
      return
    }
    e.preventDefault();
    self.__doTouchStart(e, e.timeStamp);
  };

  var touchMoveHandler = function (e) {
    self.__doTouchMove(e, e.timeStamp);
  };

  var touchEndHandler = function (e) {
    self.__doTouchEnd(e.timeStamp);
  };

  component.addEventListener('touchstart', touchStartHandler, false);
  component.addEventListener('mousedown', touchStartHandler, false);

  component.addEventListener('touchmove', touchMoveHandler, false);
  component.addEventListener('mousemove', touchMoveHandler, false);

  component.addEventListener('touchend', touchEndHandler, false);
  component.addEventListener('mouseup', touchEndHandler, false);
};

var members = {
  value: null,
  __prevValue: null,
  __isSingleTouch: false,
  __isTracking: false,
  __didDecelerationComplete: false,
  __isGesturing: false,
  __isDragging: false,
  __isDecelerating: false,
  __isAnimating: false,
  __clientTop: 0,
  __clientHeight: 0,
  __contentHeight: 0,
  __itemHeight: 0,
  __scrollTop: 0,
  __minScrollTop: 0,
  __maxScrollTop: 0,
  __scheduledTop: 0,
  __lastTouchTop: null,
  __lastTouchMove: null,
  __positions: null,
  __minDecelerationScrollTop: null,
  __maxDecelerationScrollTop: null,
  __decelerationVelocityY: null,

  __setDimensions: function __setDimensions (clientHeight, contentHeight) {
    var self = this;

    self.__clientHeight = clientHeight;
    self.__contentHeight = contentHeight;

    var totalItemCount = self.options.data.length;
    var clientItemCount = Math.round(self.__clientHeight / self.__itemHeight);

    self.__minScrollTop = -self.__itemHeight * (clientItemCount / 2);
    self.__maxScrollTop = self.__minScrollTop + totalItemCount * self.__itemHeight - 0.1;
  },

  selectByIndex: function selectByIndex (index, animate) {
    var self = this;
    if (index < 0 || index > self.__content.childElementCount - 1) {
      return
    }
    self.__scrollTop = self.__minScrollTop + index * self.__itemHeight;

    self.scrollTo(self.__scrollTop, animate);

    self.__selectItem(self.__content.children[index]);
  },

  select: function select (value, animate) {
    var self = this;

    var children = self.__content.children;
    for (var i = 0, len = children.length; i < len; i++) {
      if (decodeValue(children[i].dataset.value) === value) {
        self.selectByIndex(i, animate);
        return
      }
    }

    self.selectByIndex(0, animate);
  },

  getValue: function getValue () {
    return this.value
  },

  scrollTo: function scrollTo (top, animate) {
    var self = this;

    animate = (animate === undefined) ? true : animate;

    if (self.__isDecelerating) {
      Animate.stop(self.__isDecelerating);
      self.__isDecelerating = false;
    }

    top = Math.round(top / self.__itemHeight) * self.__itemHeight;
    top = Math.max(Math.min(self.__maxScrollTop, top), self.__minScrollTop);

    if (top === self.__scrollTop || !animate) {
      self.__publish(top);
      self.__scrollingComplete();
      return
    }
    self.__publish(top, 250);
  },

  destroy: function destroy () {
    this.__component.parentNode && this.__component.parentNode.removeChild(this.__component);
  },

  __selectItem: function __selectItem (selectedItem) {
    var self = this;

    var selectedItemClass = self.options.itemClass + '-selected';
    var lastSelectedElem = self.__content.querySelector('.' + selectedItemClass);
    if (lastSelectedElem) {
      lastSelectedElem.classList.remove(selectedItemClass);
    }
    selectedItem.classList.add(selectedItemClass);

    if (self.value !== null) {
      self.__prevValue = self.value;
    }

    self.value = decodeValue(selectedItem.dataset.value); // decodeURI(JSON.parse(selectedItem.dataset.value).value)
  },

  __scrollingComplete: function __scrollingComplete () {
    var self = this;

    var index = Math.round((self.__scrollTop - self.__minScrollTop - self.__itemHeight / 2) / self.__itemHeight);

    self.__selectItem(self.__content.children[index]);

    if (self.__prevValue !== null && self.__prevValue !== self.value) {
      self.options.onSelect(self.value, index);
    }
  },

  __doTouchStart: function __doTouchStart (ev, timeStamp) {
    var touches = ev.touches;
    var self = this;
    var target = ev.touches ? ev.touches[0] : ev;
    var isMobile = !!ev.touches;

    self.options.onStart();

    if (ev.touches && touches.length == null) {
      throw new Error('Invalid touch list: ' + touches)
    }
    if (timeStamp instanceof Date) {
      timeStamp = timeStamp.valueOf();
    }
    if (typeof timeStamp !== 'number') {
      throw new Error('Invalid timestamp value: ' + timeStamp)
    }

    self.__interruptedAnimation = true;

    if (self.__isDecelerating) {
      Animate.stop(self.__isDecelerating);
      self.__isDecelerating = false;
      self.__interruptedAnimation = true;
    }

    if (self.__isAnimating) {
      Animate.stop(self.__isAnimating);
      self.__isAnimating = false;
      self.__interruptedAnimation = true;
    }

    // Use center point when dealing with two fingers
    var currentTouchTop;
    var isSingleTouch = (isMobile && touches.length === 1) || !isMobile;
    if (isSingleTouch) {
      currentTouchTop = target.pageY;
    } else {
      currentTouchTop = Math.abs(target.pageY + touches[1].pageY) / 2;
    }

    self.__initialTouchTop = currentTouchTop;
    self.__lastTouchTop = currentTouchTop;
    self.__lastTouchMove = timeStamp;
    self.__lastScale = 1;
    self.__enableScrollY = !isSingleTouch;
    self.__isTracking = true;
    self.__didDecelerationComplete = false;
    self.__isDragging = !isSingleTouch;
    self.__isSingleTouch = isSingleTouch;
    self.__positions = [];
  },

  __doTouchMove: function __doTouchMove (ev, timeStamp, scale) {
    var self = this;
    var touches = ev.touches;
    var target = ev.touches ? ev.touches[0] : ev;
    var isMobile = !!ev.touches;

    if (touches && touches.length == null) {
      throw new Error('Invalid touch list: ' + touches)
    }
    if (timeStamp instanceof Date) {
      timeStamp = timeStamp.valueOf();
    }
    if (typeof timeStamp !== 'number') {
      throw new Error('Invalid timestamp value: ' + timeStamp)
    }

    // Ignore event when tracking is not enabled (event might be outside of element)
    if (!self.__isTracking) {
      return
    }

    var currentTouchTop;

    // Compute move based around of center of fingers
    if (isMobile && touches.length === 2) {
      currentTouchTop = Math.abs(target.pageY + touches[1].pageY) / 2;
    } else {
      currentTouchTop = target.pageY;
    }

    var positions = self.__positions;

    // Are we already is dragging mode?
    if (self.__isDragging) {
      var moveY = currentTouchTop - self.__lastTouchTop;
      var scrollTop = self.__scrollTop;

      if (self.__enableScrollY) {
        scrollTop -= moveY;

        var minScrollTop = self.__minScrollTop;
        var maxScrollTop = self.__maxScrollTop;

        if (scrollTop > maxScrollTop || scrollTop < minScrollTop) {
          // Slow down on the edges
          if (scrollTop > maxScrollTop) {
            scrollTop = maxScrollTop;
          } else {
            scrollTop = minScrollTop;
          }
        }
      }

      // Keep list from growing infinitely (holding min 10, max 20 measure points)
      if (positions.length > 40) {
        positions.splice(0, 20);
      }

      // Track scroll movement for decleration
      positions.push(scrollTop, timeStamp);

      // Sync scroll position
      self.__publish(scrollTop);

      // Otherwise figure out whether we are switching into dragging mode now.
    } else {
      var minimumTrackingForScroll = 0;
      var minimumTrackingForDrag = 5;

      var distanceY = Math.abs(currentTouchTop - self.__initialTouchTop);

      self.__enableScrollY = distanceY >= minimumTrackingForScroll;

      positions.push(self.__scrollTop, timeStamp);

      self.__isDragging = self.__enableScrollY && (distanceY >= minimumTrackingForDrag);

      if (self.__isDragging) {
        self.__interruptedAnimation = false;
      }
    }

    // Update last touch positions and time stamp for next event
    self.__lastTouchTop = currentTouchTop;
    self.__lastTouchMove = timeStamp;
    self.__lastScale = scale;
  },

  __doTouchEnd: function __doTouchEnd (timeStamp) {
    var self = this;

    self.options.onEnd();

    if (timeStamp instanceof Date) {
      timeStamp = timeStamp.valueOf();
    }
    if (typeof timeStamp !== 'number') {
      throw new Error('Invalid timestamp value: ' + timeStamp)
    }

    // Ignore event when tracking is not enabled (no touchstart event on element)
    // This is required as this listener ('touchmove') sits on the document and not on the element itself.
    if (!self.__isTracking) {
      return
    }

    // Not touching anymore (when two finger hit the screen there are two touch end events)
    self.__isTracking = false;

    // Be sure to reset the dragging flag now. Here we also detect whether
    // the finger has moved fast enough to switch into a deceleration animation.
    if (self.__isDragging) {
      // Reset dragging flag
      self.__isDragging = false;

      // Start deceleration
      // Verify that the last move detected was in some relevant time frame
      if (self.__isSingleTouch && (timeStamp - self.__lastTouchMove) <= 100) {
        // Then figure out what the scroll position was about 100ms ago
        var positions = self.__positions;
        var endPos = positions.length - 1;
        var startPos = endPos;

        // Move pointer to position measured 100ms ago
        for (var i = endPos; i > 0 && positions[i] > (self.__lastTouchMove - 100); i -= 2) {
          startPos = i;
        }

        // If start and stop position is identical in a 100ms timeframe,
        // we cannot compute any useful deceleration.
        if (startPos !== endPos) {
          // Compute relative movement between these two points
          var timeOffset = positions[endPos] - positions[startPos];
          var movedTop = self.__scrollTop - positions[startPos - 1];

          // Based on 50ms compute the movement to apply for each render step
          self.__decelerationVelocityY = movedTop / timeOffset * (1000 / 60);

          // How much velocity is required to start the deceleration
          var minVelocityToStartDeceleration = 4;

          // Verify that we have enough velocity to start deceleration
          if (Math.abs(self.__decelerationVelocityY) > minVelocityToStartDeceleration) {
            self.__startDeceleration(timeStamp);
          }
        }
      }
    }

    if (!self.__isDecelerating) {
      self.scrollTo(self.__scrollTop);
    }

    // Fully cleanup list
    self.__positions.length = 0;
  },

  // Applies the scroll position to the content element
  __publish: function __publish (top, animationDuration) {
    var self = this;

    // Remember whether we had an animation, then we try to continue based on the current "drive" of the animation
    var wasAnimating = self.__isAnimating;
    if (wasAnimating) {
      Animate.stop(wasAnimating);
      self.__isAnimating = false;
    }

    if (animationDuration) {
      // Keep scheduled positions for scrollBy functionality
      self.__scheduledTop = top;

      var oldTop = self.__scrollTop;
      var diffTop = top - oldTop;

      var step = function (percent, now, render) {
        self.__scrollTop = oldTop + (diffTop * percent);
        // Push values out
        if (self.__callback) {
          self.__callback(self.__scrollTop);
        }
      };

      var verify = function (id) {
        return self.__isAnimating === id
      };

      var completed = function (renderedFramesPerSecond, animationId, wasFinished) {
        if (animationId === self.__isAnimating) {
          self.__isAnimating = false;
        }
        if (self.__didDecelerationComplete || wasFinished) {
          self.__scrollingComplete();
        }
      };

      // When continuing based on previous animation we choose an ease-out animation instead of ease-in-out
      self.__isAnimating = Animate.start(step, verify, completed, animationDuration, wasAnimating ? easeOutCubic : easeInOutCubic);
    } else {
      self.__scheduledTop = self.__scrollTop = top;
      // Push values out
      if (self.__callback) {
        self.__callback(top);
      }
    }
  },

  // Called when a touch sequence end and the speed of the finger was high enough to switch into deceleration mode.
  __startDeceleration: function __startDeceleration (timeStamp) {
    var self = this;

    self.options.onStart();

    self.__minDecelerationScrollTop = self.__minScrollTop;
    self.__maxDecelerationScrollTop = self.__maxScrollTop;

    // Wrap class method
    var step = function (percent, now, render) {
      self.__stepThroughDeceleration(render);
    };

    // How much velocity is required to keep the deceleration running
    var minVelocityToKeepDecelerating = 0.5;

    // Detect whether it's still worth to continue animating steps
    // If we are already slow enough to not being user perceivable anymore, we stop the whole process here.
    var verify = function () {
      var shouldContinue = Math.abs(self.__decelerationVelocityY) >= minVelocityToKeepDecelerating;
      if (!shouldContinue) {
        self.__didDecelerationComplete = true;
      }
      return shouldContinue
    };

    var completed = function (renderedFramesPerSecond, animationId, wasFinished) {
      self.__isDecelerating = false;
      if (self.__scrollTop <= self.__minScrollTop || self.__scrollTop >= self.__maxScrollTop) {
        self.scrollTo(self.__scrollTop);
        return
      }
      if (self.__didDecelerationComplete) {
        self.__scrollingComplete();
      }
    };

    // Start animation and switch on flag
    self.__isDecelerating = Animate.start(step, verify, completed);
  },

  // Called on every step of the animation
  __stepThroughDeceleration: function __stepThroughDeceleration (render) {
    var self = this;

    var scrollTop = self.__scrollTop + self.__decelerationVelocityY;

    var scrollTopFixed = Math.max(Math.min(self.__maxDecelerationScrollTop, scrollTop), self.__minDecelerationScrollTop);
    if (scrollTopFixed !== scrollTop) {
      scrollTop = scrollTopFixed;
      self.__decelerationVelocityY = 0;
    }

    if (Math.abs(self.__decelerationVelocityY) <= 1) {
      if (Math.abs(scrollTop % self.__itemHeight) < 1) {
        self.__decelerationVelocityY = 0;
      }
    } else {
      self.__decelerationVelocityY *= 0.95;
    }

    self.__publish(scrollTop);
  }
};

// Copy over members to prototype
for (var key in members) {
  Scroller.prototype[key] = members[key];
}

var Manager = (function () {
  function Manager (data, count, fixedColumns) {
    this.data = data;
    this.count = count;
    if (fixedColumns) {
      this.fixedColumns = fixedColumns;
    }
  }

  Manager.prototype.getChildren = function getChildren (value) {
    return filter(this.data, function (one) {
      return one.parent === value
    })
  };

  Manager.prototype.getFirstColumn = function getFirstColumn () {
    return filter(this.data, function (one) {
      return !one.parent || one.parent === 0 || one.parent === '0'
    })
  };

  Manager.prototype.getPure = function getPure (obj) {
    return JSON.parse(JSON.stringify(obj))
  };

  Manager.prototype.getColumns = function getColumns (value) {
    var this$1 = this;

    // check is data contains the values
    if (value.length > 0) {
      var matchCount = this.getPure(this.data).filter(function (item) {
        return this$1.getPure(value).indexOf(item.value) > -1
      }).length;
      if (matchCount < this.getPure(value).length) {
        value = [];
      }
    }
    var datas = [];
    var max = this.fixedColumns || 8;
    for (var i = 0; i < max; i++) {
      if (i === 0) {
        datas.push(this$1.getFirstColumn());
      } else {
        // 没有数据时，取得上一级的第一个
        if (!value[i]) {
          if (typeof datas[i - 1][0] === 'undefined') {
            break
          } else {
            var topValue = datas[i - 1][0].value;
            datas.push(this$1.getChildren(topValue));
          }
        } else {
          datas.push(this$1.getChildren(value[i - 1]));
        }
      }
    }
    var list = datas.filter(function (item) {
      return item.length > 0
    });
    // correct the column
    this.count = list.length;
    return list
  };

  return Manager;
}());

var PickerView = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{staticClass:"ui-picker"},[_c('flexbox',{attrs:{"gutter":0}},_vm._l((_vm.currentData),function(one,index){return _c('flexbox-item',{key:index,staticStyle:{"margin-left":"0"},attrs:{"span":_vm.columnWidth && _vm.columnWidth[index]}},[_c('div',{staticClass:"ui-picker-item",attrs:{"id":("ui-picker-" + (_vm.uuid) + "-" + index)}})])}))],1)},staticRenderFns: [],
  components: {
    Flexbox: Flexbox,
    FlexboxItem: FlexboxItem
  },
  created: function created () {
    if (this.columns !== 0) {
      var length = this.columns;

      this.store = new Manager(this.data, length, this.fixedColumns);
      this.currentData = this.store.getColumns(this.value);
    }
  },
  mounted: function mounted () {
    var this$1 = this;

    this.uuid = generateUUID();
    this.$nextTick(function () {
      this$1.render(this$1.currentData, this$1.currentValue);
    });
  },
  props: {
    data: [Array],
    columns: {
      type: Number,
      default: 0
    },
    fixedColumns: {
      type: Number,
      default: 0
    },
    value: [Number, String, Array],
    itemClass: {
      type: String,
      default: 'scroller-item'
    },
    columnWidth: Array,
    rangeKey: {
      type: String,
      default: 'name'
    }
  },
  methods: {
    getNameValues: function getNameValues () {
      return value2name(this.currentValue, this.data)
    },
    getId: function getId (i) {
      return ("#ui-picker-" + (this.uuid) + "-" + i)
    },
    render: function render (data, value) {
      var this$1 = this;

      this.count = this.currentData.length;
      var _this = this;
      if (!data || !data.length) {
        return
      }
      var count = this.currentData.length;
      // set first item as value
      if (value.length < count) {
        for (var i = 0; i < count; i++) {
          this$1.$set(_this.currentValue, i, data[i][0].value || data[i][0]);
        }
      }

      var loop = function ( i ) {
        /**
        * Still don't know why this happens
        */
        if (!document.querySelector(_this.getId(i))) {
          return {}
        }

        _this.scroller[i] && _this.scroller[i].destroy();
        _this.scroller[i] = new Scroller(_this.getId(i), {
          data: data[i],
          defaultValue: value[i] || data[i][0].value,
          itemClass: _this.item_class,
          rangeKey: this$1.rangeKey,
          onStart: function onStart () {
            _this.$emit('on-start');
          },
          onEnd: function onEnd () {
            _this.$emit('on-end');
          },
          onSelect: function onSelect (value, index) {
            _this.$set(_this.currentValue, i, value);

            if (!this.columns || (this.columns && _this.getValue().length === _this.store.count)) {
              _this.$emit('on-change', _this.getValue());
            }
            if (_this.columns !== 0) {
              _this.renderChain(i + 1);
            }
          }
        });
        if (_this.currentValue) {
          _this.scroller[i].select(value[i]);
        }
      };

      for (var i$1 = 0; i$1 < data.length; i$1++) {
        var returned = loop( i$1 );

        if ( returned ) return returned.v;
      }
    },
    renderChain: function renderChain (i) {
      if (!this.columns) {
        return
      }

      // do not render for last scroller
      if (i > this.count - 1) {
        return
      }

      var _this = this;
      var ID = this.getId(i);
      // destroy old one
      this.scroller[i].destroy();
      var list = this.store.getChildren(_this.getValue()[i - 1]);
      this.scroller[i] = new Scroller(ID, {
        data: list,
        itemClass: _this.item_class,
        rangeKey: this.rangeKey,
        onStart: function onStart () {
          _this.$emit('on-start');
        },
        onEnd: function onEnd () {
          _this.$emit('on-end');
        },
        onSelect: function onSelect (value) {
          _this.$set(_this.currentValue, i, value);
          _this.$emit('on-change', _this.getValue());
          _this.renderChain(i + 1);
        }
      });
      this.$set(this.currentValue, i, list[0].value);
      this.renderChain(i + 1);
    },
    getValue: function getValue () {
      var this$1 = this;

      var data = [];
      for (var i = 0; i < this.currentData.length; i++) {
        if (this$1.scroller[i]) {
          data.push(this$1.scroller[i].value);
        } else {
          return []
        }
      }
      return data
    },
    emitValueChange: function emitValueChange (val) {
      if (!this.columns || (this.columns && val.length === this.store.count)) {
        this.$emit('on-change', val);
      }
    }
  },
  data: function data () {
    return {
      scroller: [],
      count: 0,
      uuid: '',
      currentData: this.data,
      currentValue: this.value
    }
  },
  watch: {
    value: function value (val) {
      if (JSON.stringify(val) !== JSON.stringify(this.currentValue)) {
        this.currentValue = val;
      }
    },
    currentValue: function currentValue (val, oldVal) {
      var this$1 = this;

      this.$emit('input', val);
      // render all the scroller for chain datas
      if (this.columns !== 0) {
        if (val.length > 0) {
          if (JSON.stringify(val) !== JSON.stringify(oldVal)) {
            this.currentData = this.store.getColumns(val);
            this.$nextTick(function () {
              this.render(this.currentData, val);
            });
          }
        }
      } else {
        if (val.length) {
          for (var i = 0; i < val.length; i++) {
            if (this$1.scroller[i] && this$1.scroller[i].value !== val[i]) {
              this$1.scroller[i].select(val[i]);
            }
          }
        } else {
          this.render(this.currentData, val);
        }
      }
    },
    data: function data (val) {
      if (JSON.stringify(val) !== JSON.stringify(this.currentData)) {
        this.currentData = val;
      }
    },
    currentData: function currentData (newData) {
      var this$1 = this;

      if (Object.prototype.toString.call(newData[0]) === '[object Array]') {
        this.$nextTick(function () {
          this$1.render(newData, this$1.currentValue);
          // emit on-change after rerender
          this$1.$nextTick(function () {
            this$1.emitValueChange(this$1.getValue());

            if (JSON.stringify(this$1.getValue()) !== JSON.stringify(this$1.currentValue)) {
              if (!this$1.columns || (this$1.columns && this$1.getValue().length === this$1.store.count)) {
                this$1.currentValue = this$1.getValue();
              }
            }
          });
        });
      } else {
        if (this.columns !== 0) {
          if (!newData.length) {
            return
          }
          var length = this.columns;
          this.store = new Manager(newData, length, this.fixedColumns);
          this.currentData = this.store.getColumns(this.currentValue);
        }
      }
    }
  },
  beforeDestroy: function beforeDestroy () {
    var this$1 = this;

    for (var i = 0; i < this.count; i++) {
      this$1.scroller[i] && this$1.scroller[i].destroy();
      this$1.scroller[i] = null;
    }
  }
};

var getObject = function (obj) {
  return JSON.parse(JSON.stringify(obj))
};

var Picker = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{staticClass:"ui-cell-box"},[_c('div',{directives:[{name:"show",rawName:"v-show",value:(_vm.showCell),expression:"showCell"}],staticClass:"weui-cell ui-tap-active weui-cell_access"},[_c('div',{staticClass:"ui-cell-primary ui-picker-select-box"},[_c('div',{staticClass:"ui-picker-select",style:({textAlign: _vm.valueTextAlign})},[(!_vm.displayFormat && !_vm.showName && _vm.value.length)?_c('span',{staticClass:"ui-popup-picker-value"},[_vm._v(_vm._s(_vm._f("array2string")(_vm.value)))]):_vm._e(),_vm._v(" "),(!_vm.displayFormat && _vm.showName && _vm.value.length)?_c('span',{staticClass:"ui-popup-picker-value"},[_vm._v(_vm._s(_vm._f("value2name")(_vm.value,_vm.range, _vm.delimiter)))]):_vm._e(),_vm._v(" "),(_vm.displayFormat && _vm.value.length)?_c('span',{staticClass:"ui-popup-picker-value"},[_vm._v(_vm._s(_vm.displayFormat(_vm.value, _vm.value2name(_vm.value, _vm.range))))]):_vm._e(),_vm._v(" "),(!_vm.value.length && _vm.placeholder)?_c('span',{staticClass:"ui-picker-placeholder",domProps:{"textContent":_vm._s(_vm.placeholder)}}):_vm._e()])])]),_vm._v(" "),_c('picker-view',{attrs:{"data":_vm.range,"columns":_vm.columns,"fixed-columns":_vm.fixedColumns,"container":'#ui-popup-picker-'+_vm.uuid,"column-width":_vm.columnWidth},on:{"on-change":_vm.onPickerChange},model:{value:(_vm.tempValue),callback:function ($$v) {_vm.tempValue=$$v;},expression:"tempValue"}})],1)},staticRenderFns: [],
  created: function created () {
    if (typeof this.show !== 'undefined') {
      this.showValue = this.show;
    }
  },
  mixins: [uuidMixin],
  components: {
    PickerView: PickerView,
    Cell: Cell,
    Popup: Popup,
    Flexbox: Flexbox,
    FlexboxItem: FlexboxItem,
    InlineDesc: InlineDesc
  },
  filters: {
    array2string: array2string,
    value2name: value2name
  },
  props: {
    range: {
      type: Array,
      default: Array
    },
    value: {
      type: [Number, String, Array]
    },
    valueTextAlign: {
      type: String,
      default: 'center'
    },
    title: String,
    cancelText: String,
    confirmText: String,
    placeholder: String,
    columns: {
      type: Number,
      default: 0
    },
    fixedColumns: {
      type: Number,
      default: 0
    },
    showName: Boolean,
    inlineDesc: [String, Number, Array, Object, Boolean],
    showCell: {
      type: Boolean,
      default: true
    },
    show: Boolean,
    displayFormat: Function,
    isTransferDom: {
      type: Boolean,
      default: true
    },
    columnWidth: Array,
    delimiter: {
      type: String,
      default: ' '
    }
  },
  computed: {
    labelStyles: function labelStyles () {
      return {
        display: 'block',
        width: this.$parent.labelWidth || this.$parent.$parent.labelWidth,
        textAlign: this.$parent.labelAlign || this.$parent.$parent.labelAlign,
        marginRight: this.$parent.labelMarginRight || this.$parent.$parent.labelMarginRight
      }
    }
  },
  methods: {
    value2name: value2name,
    getNameValues: function getNameValues () {
      return value2name(this.currentValue, this.range)
    },
    onClick: function onClick () {
      this.showValue = true;
    },
    onPickerChange: function onPickerChange (val) {
      var this$1 = this;

      if (JSON.stringify(this.currentValue) !== JSON.stringify(val)) {
        // if has value, replace it
        if (this.value.length) {
          var nowData = JSON.stringify(this.range);
          if (nowData !== this.currentData && this.currentData !== '[]') {
            this.tempValue = getObject(val);
          }
          this.currentData = nowData;

          this.$nextTick(function () {
            this$1.currentValue = getObject(this$1.tempValue);
          });
        } else { // if no value, stay quiet
          // if set to auto update, do update the value
        }
      }
      // TODO: bindcolumnchange
      this.$emit('on-shadow-change', getObject(val));
    }
  },
  watch: {
    value: function value (val, oldVal) {
      if (JSON.stringify(val) !== JSON.stringify(this.tempValue)) {
        this.tempValue = getObject(val);
      }

      if (JSON.stringify(val) !== JSON.stringify(oldVal)) {
        this.$emit('on-change', getObject(val));
      }
    },
    currentValue: function currentValue (val) {
      // this.$emit('on-change', getObject(val))
      this.$emit('input', getObject(val));
    },
    show: function show (val) {
      this.showValue = val;
    }
  },
  data: function data () {
    return {
      onShowProcess: false,
      tempValue: getObject(this.value),
      closeType: false,
      currentData: JSON.stringify(this.range), // used for detecting if it is after data change
      showValue: false,
      currentValue: this.value,
      canConfirm: true
    }
  }
};

var selector = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('picker',{attrs:{"title":_vm.title,"range":_vm.selfRange},on:{"on-change":_vm.onChange},model:{value:(_vm.selfValue),callback:function ($$v) {_vm.selfValue=$$v;},expression:"selfValue"}})},staticRenderFns: [],
  props: {
    range: {
      type: Array,
      default: Array
    },
    title: {
      type: String,
      default: '当前选择'
    },
    value: {
      type: Number,
      default: 0
    }
  },
  components: {
    picker: Picker
  },
  data: function data () {
    return {
      selfRange: [this.range],
      selfValue: [this.range[this.value]]
    }
  },
  watch: {
    value: function value () {
      this.selfValue = [this.range[this.value]];
    },
    range: function range () {
      if (Array.isArray(this.range)) {
        this.selfRange = [this.range];
      }
    }
  },
  methods: {
    onChange: function onChange (val) {
      this.$emit('change', this.range.indexOf(val[0]));
    }
  }
};

var multiSelector = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return (_vm.isChain)?_c('picker',{attrs:{"title":_vm.title,"range":_vm.selfRange,"columns":_vm.columns,"show-name":""},on:{"on-change":_vm.onChange},model:{value:(_vm.selfValue),callback:function ($$v) {_vm.selfValue=$$v;},expression:"selfValue"}}):_c('picker',{attrs:{"title":_vm.title,"range":_vm.selfRange},on:{"on-change":_vm.onChange},model:{value:(_vm.selfValue),callback:function ($$v) {_vm.selfValue=$$v;},expression:"selfValue"}})},staticRenderFns: [],
  props: {
    range: {
      type: Array,
      default: Array
    },
    title: {
      type: String,
      default: '当前选择'
    },
    value: {
      type: Array
    },
    isChain: Boolean
  },
  components: {
    picker: Picker
  },
  data: function data () {
    return {
      selfRange: [],
      selfValue: [],
      columns: this.getColumns()
    }
  },
  watch: {
    value: function value () {
      this.selfValue = this.getSelfValue();
    },
    range: function range () {
      if (Array.isArray(this.range)) {
        this.selfRange = this.getSelfRange();
      }
    }
  },
  mounted: function mounted () {
    this.selfRange = this.getSelfRange();
    this.selfValue = this.getSelfValue();
  },
  methods: {
    getSelfRange: function getSelfRange () {
      if (this.isChain) {
        return this.flattenArray(this.range)
      } else {
        return this.range
      }
    },
    getSelfValue: function getSelfValue () {
      var this$1 = this;

      if (this.value) {
        if (this.isChain) {
          return this.value
        } else {
          var selfValue = [];
          for (var i = 0; i < this.value.length; i++) {
            selfValue.push(this$1.range[i][this$1.value[i]]);
          }
          return selfValue
        }
      } else {
        var selfValue$1 = [];
        if (this.isChain) {
          var start = this.range[0];
          selfValue$1.push(start.value);
          while (start.children) {
            start = start.children[0];
            selfValue$1.push(start.value);
          }
          return selfValue$1
        } else {
          selfValue$1 = this.range.map(function (item) {
            return item[0]
          });
        }

        return selfValue$1
      }
    },
    flattenArray: function flattenArray (array, parent) {
      var this$1 = this;

      var result = [];
      for (var i = 0; i < array.length; i++) {
        if (parent) {
          result.push({
            name: array[i].name,
            value: array[i].value,
            parent: parent
          });
        } else {
          result.push({
            name: array[i].name,
            value: array[i].value
          });
        }
        if (array[i].children) {
          result = result.concat(this$1.flattenArray(array[i].children, array[i].value));
        }
      }
      return result
    },
    // 在isChain模式下，需要知道columns值
    getColumns: function getColumns () {
      var columns = 1;
      var start = this.range[0];
      while (start.children) {
        columns++;
        start = start.children[0];
      }
      return columns
    },
    getValue: function getValue (val) {
      var this$1 = this;

      if (this.isChain) {
        return val
      } else {
        var value = [];
        for (var i = 0; i < val.length; i++) {
          value.push(this$1.range[i].indexOf(val[i]));
        }
        return value
      }
    },
    onChange: function onChange (val) {
      this.$emit('change', this.getValue(val));
    }
  }
};

var timeSelector = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('picker',{attrs:{"title":_vm.title,"range":_vm.selfRange,"display-format":_vm.format},on:{"on-change":_vm.onChange},model:{value:(_vm.selfValue),callback:function ($$v) {_vm.selfValue=$$v;},expression:"selfValue"}})},staticRenderFns: [],
  props: {
    range: {
      type: Array,
      default: Array
    },
    title: {
      type: String,
      default: '当前选择'
    },
    value: String
  },
  components: {
    picker: Picker
  },
  data: function data () {
    return {
      selfRange: this.getTime(),
      selfValue: this.getSelfValue(),
      format: function (value, name) {
        return ((value[0]) + ":" + (value[1]))
      }
    }
  },
  watch: {
    value: function value () {
      this.selfValue = this.getSelfValue();
    }
  },
  methods: {
    getTime: function getTime () {
      var time = [];
      var hours = [];
      var minutes = [];
      for (var i = 0; i < 24; i++) {
        hours.push(i < 10 ? '0' + i : i + '');
      }
      for (var j = 0; j < 60; j++) {
        minutes.push(j < 10 ? '0' + j : j + '');
      }
      time.push(hours);
      time.push(minutes);

      return time
    },
    getSelfValue: function getSelfValue () {
      return this.value.split(':')
    },
    getValue: function getValue (val) {
      return val
    },
    onChange: function onChange (val) {
      this.$emit('change', val.join(':'));
    }
  }
};

function padZero (num, len) {
  var zero = '';
  for (var i = 0; i < len - num.toString().length; i++) {
    zero += '0';
  }
  return zero + num
}

function padZeroForArray (arr, len) {
  return arr.map(function (item) {
    return padZero(item, len)
  })
}

var util$1 = {
  // '2017-09-01' => new Date(2017, 8, 1)
  str2Date: function str2Date (str) {
    var arr = str.split('-');
    if (arr.length === 3) {
      return new Date(arr[0], Number(arr[1] - 1), arr[2])
    }
  },
  // new Date(2017, 8, 1) => ['2017', '09', '01']
  date2Arr: function date2Arr (date) {
    var year = date.getFullYear();
    var month = date.getMonth() + 1;
    var day = date.getDate();
    return padZeroForArray([year, month, day], 2)
  },
  // new Date(2017, 8, 1) => '2017-09-01'
  date2Str: function date2Str (date) {
    return this.date2Arr(date).join('-')
  },
  // '2017-09-01', { datepart: 'year', value: 1 } => '2018-09-01'
  dateStrAdd: function dateStrAdd (str, ref) {
    var datepart = ref.datepart;
    var value = ref.value;

    var date = this.str2Date(str);
    if (datepart === 'year') {
      date.setYear(date.getFullYear() + value);
    } else if (datepart === 'month') {
      date.setMonth(date.getMonth() + value);
    } else if (datepart === 'date') {
      date.setMonth(date.getDate() + value);
    }
    return this.date2Str(date)
  },
  generateTimeList: function generateTimeList (start, end) {
    var time = [];

    var startParts = start.split(':').map(function (p) { return Number(p); });
    var startHour = startParts[0];
    var startMinute = startParts[1];

    var endParts = end.split(':').map(function (p) { return Number(p); });
    var endHour = endParts[0];
    var endMinute = endParts[1];

    var hour = '';
    var minute = '';

    for (var i = startHour; i <= endHour; i++) {
      hour = padZero(i, 2);

      time.push({
        name: hour,
        value: hour,
        parent: 0
      });

      var ks = 0;
      var ke = 59;

      if (i === startHour) {
        ks = startMinute;
      }
      if (i === endHour) {
        ke = endMinute;
      }

      for (var j = ks; j <= ke; j++) {
        minute = padZero(j, 2);
        time.push({
          name: minute,
          value: hour + ':' + minute,
          parent: hour
        });
      }
    }

    return time
  },
  generateDateList: function generateDateList (start, end) {
    var this$1 = this;

    var years = [];
    var months = [];
    var days = [];

    var yearValues = [];
    var monthValues = [];

    var startDate = fecha.parse(start, 'YYYY-MM-DD');
    var endDate = fecha.parse(end, 'YYYY-MM-DD');

    var startTime = startDate.getTime();
    var diff = (endDate - startDate) / 86400000;

    var curDate, dayName, monthName, dayValue, dayParts, dayParentValue;

    for (var i = 0; i <= diff; i++) {
      curDate = new Date(startTime + 86400000 * i);
      dayName = padZero(curDate.getDate(), 2);
      monthName = padZero(curDate.getMonth() + 1, 2);

      dayParts = this$1.date2Arr(curDate);
      dayValue = dayParts.join('-');
      dayParentValue = dayParts[0] + '-' + dayParts[1];

      days.push({
        name: dayName,
        value: dayValue,
        parent: dayParentValue
      });

      if (monthValues.indexOf(dayParentValue) < 0) {
        monthValues.push(dayParentValue);
        months.push({
          name: monthName,
          value: dayParentValue,
          parent: dayParts[0]
        });

        if (yearValues.indexOf(dayParts[0]) < 0) {
          yearValues.push(dayParts[0]);
          years.push({
            name: dayParts[0],
            value: dayParts[0],
            parent: 0
          });
        }
      }
    }

    return [].concat(years, months, days)
  }
};

var dateSelector = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('picker',{attrs:{"title":_vm.title,"range":_vm.selfRange,"columns":3,"display-format":_vm.displayFormat,"show-name":""},on:{"on-change":_vm.onChange},model:{value:(_vm.selfValue),callback:function ($$v) {_vm.selfValue=$$v;},expression:"selfValue"}})},staticRenderFns: [],
  props: {
    range: {
      type: Array,
      default: Array
    },
    title: {
      type: String,
      default: '当前选择'
    },
    value: String,
    chinese: Boolean,
    dateFormat: {
      type: String,
      default: 'YYYY-MM-DD'
    },
    start: {
      type: String,
      default: function () {
        return fecha.format(new Date(), 'YYYY-MM-DD')
      }
    },
    end: {
      type: String,
      default: function () {
        var date = new Date();
        return fecha.format(date.setFullYear(date.getFullYear() + 1), 'YYYY-MM-DD')
      }
    }
  },
  components: {
    picker: Picker
  },
  data: function data () {
    return {
      selfRange: [],
      selfValue: this.getSelfValue(),
      startYmd: [],
      endYmd: []
    }
  },
  watch: {
    value: function value () {
      this.selfValue = this.getSelfValue();
    }
  },
  mounted: function mounted () {
    this.selfRange = this.getDateList();
    if (this.start > this.end) {
      throw new Error('开始日期不能大于结束如期')
    }
  },
  methods: {
    getDateList: function getDateList () {
      return util$1.generateDateList(this.start, this.end)
    },
    getSelfValue: function getSelfValue () {
      var parts = this.value.split('-');
      return [parts[0], parts[0] + '-' + parts[1], parts[0] + '-' + parts[1] + '-' + parts[2]]
    },
    getValue: function getValue (val) {
      return val
    },
    displayFormat: function displayFormat (val) {
      var parts = val[2].split('-');
      var date = new Date(Number(parts[0]), Number(parts[1]) - 1, Number(parts[2]));
      return fecha.format(date, this.dateFormat)
    },
    onChange: function onChange (val) {
      this.$emit('change', val[2]);
    }
  }
};

var regionSelector = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('picker',{attrs:{"fixed-columns":_vm.hideDistrict ? 2 : 0,"columns":3,"range":_vm.addressData,"title":_vm.title,"show-name":"","inline-desc":_vm.inlineDesc,"placeholder":_vm.placeholder,"value-text-align":_vm.valueTextAlign,"confirm-text":_vm.confirmText,"cancel-text":_vm.cancelText,"display-format":_vm.displayFormat},on:{"hide":_vm.emitHide,"show":function($event){_vm.$emit('show');},"on-change":_vm.onChange},scopedSlots:_vm._u([{key:"title",fn:function(props){return [_vm._t("title",[(props.labelTitle)?_c('label',{class:props.labelClass,style:(props.labelStyle),domProps:{"innerHTML":_vm._s(props.labelTitle)}}):_vm._e()],{labelClass:props.labelClass,labelStyle:props.labelStyles,labelTitle:props.title})]}}]),model:{value:(_vm.currentValue),callback:function ($$v) {_vm.currentValue=$$v;},expression:"currentValue"}})},staticRenderFns: [],
  components: {
    Picker: Picker
  },
  props: {
    title: {
      type: String,
      required: true
    },
    range: {
      type: Array,
      default: Array
    },
    value: {
      type: Array,
      default: function default$1$$1 () {
        return []
      }
    },
    rawValue: Boolean,
    inlineDesc: String,
    placeholder: String,
    hideDistrict: Boolean,
    valueTextAlign: String,
    confirmText: String,
    cancelText: String,
    displayFormat: {
      type: Function,
      default: function (val, names) { return names; }
    }
  },

  created: function created () {
    this.addressData = this.range;
    if (this.currentValue.length && this.rawValue) {
      var parsedVal = name2value(this.currentValue, this.list);
      if (/__/.test(parsedVal)) {
        console.error('TouchUI: Wrong address value', this.currentValue);
        this.currentValue = [];
      } else {
        this.currentValue = parsedVal.split(' ');
      }
    }
  },
  mounted: function mounted () {
  },
  methods: {
    emitHide: function emitHide (val) {
      this.$emit('hide', val);
    },
    getAddressName: function getAddressName () {
      return value2name(this.value, this.list)
    },
    onChange: function onChange (val) {

    }
  },
  data: function data () {
    return {
      addressData: [],
      currentValue: this.value
    }
  },
  computed: {
    nameValue: function nameValue () {
      return value2name(this.currentValue, this.list)
    }
  },
  watch: {
    currentValue: function currentValue (val) {
      this.$emit('input', val);
    },
    value: function value (val) {
      if (val.length && !/\d+/.test(val[0])) {
        var id = name2value(val, this.list).split(' ');
        if (id[0] !== '__' && id[1] !== '__') {
          this.currentValue = id;
          return
        }
      }
      this.currentValue = val;
    }
  }
};

var index = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return (_vm.mode ==='selector')?_c('selector',{attrs:{"range":_vm.selfRange,"title":_vm.title,"value":_vm.value},on:{"change":_vm.handleChange}}):(_vm.mode ==='multiSelector')?_c('multi-selector',{attrs:{"range":_vm.selfRange,"title":_vm.title,"is-chain":_vm.isChain,"value":_vm.value},on:{"change":_vm.handleChange}}):(_vm.mode ==='time')?_c('time-selector',{attrs:{"title":_vm.title,"value":_vm.value},on:{"change":_vm.handleChange}}):(_vm.mode ==='date')?_c('date-selector',{attrs:{"start":_vm.start,"end":_vm.end,"title":_vm.title,"value":_vm.value,"date-format":_vm.dateFormat},on:{"change":_vm.handleChange}}):(_vm.mode ==='region')?_c('region-selector',{attrs:{"title":_vm.title,"range":_vm.range,"value":_vm.value,"hide-district":_vm.hideDistrict},on:{"change":_vm.handleChange}}):_vm._e()},staticRenderFns: [],
  props: {
    mode: {
      type: String,
      default: 'selector'
    },
    range: {
      type: Array,
      default: Array
    },
    title: {
      type: String,
      default: '当前选择'
    },
    value: {
      type: [Number, String, Array]
    },
    dsiabled: {
      type: Boolean,
      default: false
    },
    hideDistrict: {
      type: Boolean,
      default: false
    },
    isChain: {
      type: Boolean,
      default: false
    },
    start: {
      type: String
    },
    end: {
      type: String
    },
    dateFormat: {
      type: String,
      default: 'YYYY-MM-DD'
    }
  },
  components: {
    selector: selector,
    multiSelector: multiSelector,
    timeSelector: timeSelector,
    dateSelector: dateSelector,
    regionSelector: regionSelector
  },
  data: function data () {
    return {
      selfRange: this.range
    }
  },
  watch: {
    range: function range () {
      this.selfRange = this.range;
    }
  },
  methods: {
    handleChange: function handleChange (val) {
      this.$emit('change', val);
    }
  }
};

export default index;
